def add_function(x, y, b, m=None):
    base = {'0':0,'1':1,'2':2,'3':3,
        '4':4,'5':5,'6':6,'7':7,
        '8':8,'9':9,'a':10,'b':11,
        'c':12,'d':13,'e':14,'f':15}
    X = str(x)
    Y = str(y)
    carry = 0
    result = ''

    if x == '0' and m is None:
        return y

    if y == '0' and m is None:
        return x

    if y == '0' and x == '0':
        return '0'

    if b == 10:
        if m is None:
            return str(int(x)+int(y))
        else:
            return (int(x) + int(y)) % int(m)

    if X[0] == '-' and Y[0] == '-': #both inputs are minus, take -sign out and do normal addition
        count = 1
        X = X.replace('-','')
        Y = Y.replace('-','')
        return add_function(X,Y,b)

    if X[0] == '-':
        X = X.replace('-','')
        return subtract_function(Y,X,b)

    if Y[0] == '-':
        Y = Y.replace('-','')
        return subtract_function(X,Y,b)

    if b >= 2 and b <= 16:
        result = []
        maxL = max(len(X), len(Y))
        X = X.zfill(maxL)
        Y = Y.zfill(maxL)
        X2 = list(X)
        Y2 = list(Y)

        # maka sure the modulo has the same length as x and y
        if m is not None:
            m = m.zfill(maxL)

        # convert abcdef into integers
        for i in range(maxL):
            X2[i] = base.get(X2[i])
            Y2[i] = base.get(Y2[i])

        # primary school method of addition
        for i in range(1,maxL+1):
            dig = X2[-i] + Y2[-i] + carry
            if dig >= b:
                carry = 1
                dig %= b
            else:
                carry = 0
            result.append(dig)

        if carry == 1:
            result.append(str(carry))

        # remap the dictionary such that integers >= 10
        # are converted to alphabet
        for i in range(maxL):
            invMap = {v: k for k, v in base.items()}
            result[i] = invMap.get(result[i])

        answer = ''.join(result[::-1])

        # if m, divide by m and keep remainder as answer
        if m is not None:
            answer = simple_division(answer, m, b)[1]
        if answer[0] is "0": answer = answer[1:]
        return answer



def simple_division(x, y, b):
    #very slow for big x and small y, only used to calculate the floor of r / y*b^i in the long division function
    q, r = '0', x
    sumy = '0'
    #this covers the base case, which is also the worst case for this algorithm
    if y == '1':
        q, r = x, '0'
    #the while loop counts how many times y is in x, outputs that number (which is the quotient) and the remainder
    while subtract_function(x, add_function(sumy, y, b), b)[0] is not "-":
    #keeps track of q times y
        sumy = add_function(sumy, y, b)
    #updates the remainder
        r = str(subtract_function(r, y, b))
    #adds 1 to the quotient
        q = str(add_function(q, 1, b))

    # print(str(q) +"*" + str(y) + "+" + str(r) + "=" + str(x))
    return [q, r]



import re

from Integer import *
from algorithmsCombined import *

class Context():
    """
    """

    def __init__(self):
        self.reset()

    def reset(self):
        self.x = None
        self.y = None
        self.radix = None
        self.op = None
        self.m = None

    def read_line(self, line):
        original_line = line
        line = re.sub('\s', '', line) # Remove all whitespace

        if len(line) == 0:
            if not self.op:
                return '\n'

            if self.op in (addition, subtraction):
                computed = self.op(self.x.digits, self.x.positive, self.y.digits, self.y.positive, self.x.radix)
                answer = Integer(computed[1], self.radix, computed[0])
                self.reset()
                return f'[answer] {answer}\n\n'

            elif self.op in (multiplication, karatsuba):
                computed = self.op(self.x.digits, self.x.positive, self.y.digits, self.y.positive, self.x.radix)
                answer = Integer(computed[1], self.radix, computed[0])
                add = computed[2]
                mul = computed[3]
                self.reset()
                return f'[answer] {answer}\n[count-add] {add}\n[count-mul] {mul}\n\n'

            elif self.op in (modular_reduction, modular_inversion):
                computed = self.op(self.x.digits, self.x.positive, self.m.digits, self.x.radix)
                if isinstance(computed, str):
                    self.reset()
                    return '[answer] inverse does not exist\n\n'
                else:
                    answer = Integer(computed[1], self.radix, computed[0])
                    self.reset()
                    return f'[answer] {answer}\n\n'

            elif self.op in (modular_addition, modular_subtraction, modular_multiplication):
                computed = self.op(self.x.digits, self.x.positive, self.y.digits, self.y.positive, self.m.digits, self.x.radix)
                answer = Integer(computed[1], self.radix, computed[0])
                self.reset()
                return f'[answer] {answer}\n\n'

            elif self.op == euclidean:
                computed = self.op(self.x.digits, self.x.positive, self.y.digits, self.y.positive, self.x.radix)
                d = Integer(computed[0], self.x.radix)
                a = Integer(computed[1][1], self.x.radix, computed[1][0])
                b = Integer(computed[2][1], self.x.radix, computed[2][0])
                self.reset()
                return f'[answ-d] {d}\n[answ-a] {a}\n[answ-b] {b}\n\n'

        if line[0] == '#':
            # Line is a comment, just return it
            return original_line

        bracket_index = line.find(']')
        command = line[1:bracket_index]     # Extract text between brackets
        argument = line[bracket_index + 1:] # Extract argument after command

        if command == 'radix':
            self.radix = int(argument)

        elif command == 'x':
            self.x = Integer(argument, self.radix)

        elif command == 'y':
            self.y = Integer(argument, self.radix)

        elif command == 'm':
            self.m = Integer(argument, self.radix)

            if self.op == addition:
                self.op = modular_addition
            elif self.op == subtraction:
                self.op = modular_subtraction
            elif self.op == multiplication:
                self.op = modular_multiplication

        elif command == 'add':
            if self.m:
                self.op = modular_addition
            else:
                self.op = addition

        elif command == 'subtract':
            if self.m:
                self.op = modular_subtraction
            else:
                self.op = subtraction

        elif command == 'multiply':
            if self.m:
                self.op = modular_multiplication
            else:
                self.op = multiplication

        elif command == 'karatsuba':
            self.op = karatsuba

        elif command == 'reduce':
            self.op = modular_reduction

        elif command == 'inverse':
            self.op = modular_inversion

        elif command == 'euclid':
            self.op = euclidean

        elif command in ('answer', 'count-add', 'count-mul', 'answ-d', 'answ-a', 'answ-b'):
            # Ignore and don't write to output
            return ''

        return original_line



def reduce_function(x, m, b):
    x, m = str(x), str(m)
    x1 = x.strip('-')
    k, n = len(x1), len(m)

    for i in range(k-n, -1, -1):
        reduct_value = m
        if i == 0:
            pass
        else:
            #used to calculate m times the base to the power of i
            for j in range(0, i):
                reduct_value = reduct_value + '0'
        while subtract_function(x1, reduct_value , b)[0] is not "-":
            x1 = subtract_function(x1, reduct_value, b)

    if (x[0] is not "-") or (x1 == '0'):
        y = x1
    else:
        y = subtract_function(m, x1, b)

    return y



import argparse
import sys
from Context import *

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-i', '--input', help='name of input file')
    parser.add_argument('output', help='name of output file')

    args = parser.parse_args()

    input  = open(args.input, 'r') if args.input else sys.stdin
    output = open(args.output, 'w', encoding='utf-8')

    context = Context()

    if not args.input:
        print('Interactive mode. Ctrl-Z plus Return to exit.')

    def read_line(line, last_line = False):
        out = context.read_line(line)
        if last_line: out = out[:-1]
        output.write(out)

        if not args.input:
            print(f'OUT: {repr(out)}')

    for line in input:
        read_line(line)

    read_line('\n', True)



def subtract_function(x, y, b, m=None):
    base = {'0':0,'1':1,'2':2,'3':3,
            '4':4,'5':5,'6':6,'7':7,
            '8':8,'9':9,'a':10,'b':11,
            'c':12,'d':13,'e':14,'f':15}
    X = str(x)
    Y = str(y)
    carry = 0
    dig = 0
    maxL = max(len(X), len(Y))
    X = X.zfill(maxL)
    Y = Y.zfill(maxL)
    X2 = list(X)
    Y2 = list(Y)

    # maka sure the modulo has the same length as x and y
    if m is not None:
        m = m.zfill(maxL)

    result = []

    if x == 0 and m is None:
        return -int(y)
    if y == 0 and m is None:
        return x
    if b == 10:
        if m is None:
            return str(int(x) - int(y))
        else:
            return (int(x) - int(y)) % int(m)
    if x == y:
        return '0'

    # deal with negative numbers
    if X[0] == '-' and Y[0] == '-':
        Y = Y.replace('-','')
        return add_function(Y,X,b,m)
    if X[0] == '-':
        Y = '-' + Y
        return add_function(X,Y,b,m)
    if Y[0] == '-':
        Y = Y.replace('-','')
        return add_function(X,Y,b,m)

    if x > y:
        # convert abcdef into integers
        for i in range(maxL):
            X2[i] = base.get(X2[i])
            Y2[i] = base.get(Y2[i])

        for i in range(1,maxL+1):
            if X2[-i] >= Y2[-i]:
                dig = X2[-i] - Y2[-i]

            if X2[-i] < Y2[-i]: #take a borrow
                X2[-i-1] -= 1
                X2[-i] += b
                dig = X2[-i] - Y2[-i]
            result.append(dig)

        for i in range(maxL):
            invMap = {v: k for k, v in base.items()}
            # remap the dictionary such that integers >= 10
            # are converted to alphabet
            result[i] = invMap.get(result[i])

        if X2 > Y2:
            answer = ''.join(result[::-1])

            if m is not None:
                answer = simple_division(answer, m, b)

            if answer[0] is "0": answer = answer[1:]
            return answer
        else:
            result = subtract_function(y,x,b,m)


    if x < y:
        return '-' + subtract_function(y,x,b,m)



# All algorithms combined
def addition(inputX, positiveX, inputY, positiveY, b, countAdd = 0, countMult = 0):
    lenX = len(inputX)
    lenY = len(inputY)
    inputXY = inputX, inputY
    maxLen = max(map(len, inputXY))

    # add digits
    for digits in inputXY:
        while len(digits) <= maxLen:
            digits.insert(0, 0)

    if (positiveX == False) and (positiveY == False):
        # set positive indicator
        positiveOutput = False
        # run addition
        [output, countAdd, countMult] = addition(inputX, True, inputY, True, b, countAdd, countMult)[1:4]

    elif (positiveX == True) and (positiveY == False):
        # run subtraction
        [positiveOutput, output, countAdd, countMult] = subtraction(inputX, True, inputY, True, b, countAdd, countMult)

    elif (positiveX == False) and (positiveY == True):
        # run subtraction
        [positiveOutput, output, countAdd, countMult] = subtraction(inputY, True, inputX, True, b, countAdd, countMult)

    elif (positiveX == True) and (positiveY == True):
        # set positive indicator
        positiveOutput = True

        # sum the digits from listX with listY according to index in each list
        sumXY = [sum(x) for x in zip(*inputXY)]
        output = sumXY.copy()
        for i in range(1, len(output)):
            rem, carry = 0, 0
            if output[-i] >= b:
                carry = output[-i] // b
                output[-i] = output[-i] % b
                output[-(i+1)] += carry
                countAdd += 1

    i = 0
    while i < (len(output) - 1):
        if (output[i] == 0):
            output.pop(0)
            continue
        break

    return [positiveOutput, output, countAdd, countMult]


def subtraction(inputX, positiveX, inputY, positiveY, b, countAdd = 0, countMult = 0):
    lenX = len(inputX)
    lenY = len(inputY)
    inputXY = inputX, inputY
    maxLen = max(map(len, inputXY))

    # add digits
    for digits in inputXY:
        while len(digits) < maxLen:
            digits.insert(0, 0)

    if (positiveX == False) and (positiveY == False):
        # run addition
        [positiveOutput, output, countAdd, countMult] = subtraction(inputY, True, inputX, True, b, countAdd, countMult)

    elif (positiveX == True) and (positiveY == False):
        # run subtraction
        [positiveOutput, output, countAdd, countMult] = addition(inputX, True, inputY, True, b, countAdd, countMult)

    elif (positiveX == False) and (positiveY == True):
        # run subtraction
        [positiveOutput, output, countAdd, countMult] = addition(inputX, False, inputY, False, b, countAdd, countMult)

    elif (positiveX == True) and (positiveY == True):
        # check which number is bigger
        for i in range(maxLen):
            if (inputX[i] > inputY[i]):
                positiveOutput = True
            elif (inputX[i] < inputY[i]):
                inputTemp= inputX.copy()
                inputX = inputY.copy()
                inputY = inputTemp.copy()
                inputXY = inputX, inputY
                positiveOutput = False
            elif (inputX[i] == inputY[i]):
                if (i == maxLen - 1):
                    return [True, [0], countAdd, countMult]
                continue
            break

        # sum the digits from listX with listY according to index in each list
        diffXY = [0] * maxLen
        for i in range(maxLen):
            diffXY[i] = inputX[i] - inputY[i]
            countAdd += 1
        output = diffXY.copy()

        for i in range(1, len(output)):
            rem = 0
            if output[-i] < 0:
                output[-(i+1)] -= 1
                rem = output[-i] + b
                output[-i] = rem
                countAdd += 2

    i = 0
    while i < (len(output) - 1):
        if (output[i] == 0):
            output.pop(0)
            continue
        break

    return [positiveOutput, output, countAdd, countMult]


def multiplication(inputX, positiveX, inputY, positiveY, b, countAdd = 0, countMult = 0):
    inputXY = inputX, inputY
    maxLen = max(map(len, inputXY))
    minLen = min(map(len, inputXY))
    positiveOutput = not(positiveX ^ positiveY)

    for digits in inputXY:
        while len(digits) < maxLen:
            digits.insert(0, 0)

    n = len(inputX)
    output = [0]*2*n

    for i in range(0, n):
        digit2 = inputY[-(i+1)]
        for j in range(0, n):
            digit1 = inputX[-(j+1)]
            outputNumber = digit1 * digit2
            output[-(1+i+j)] += outputNumber
            countAdd += 1
            countMult += 1

    for i in range(1, len(output)+1):
        if output[-i] >= b:
            carry = output[-i] // b
            output[-i] = output[-i] % b
            output[-(i+1)] += carry
            countAdd += 1

    i = 0
    while i < (len(output) - 1):
        if (output[i] == 0):
            output.pop(0)
            continue
        break

    return [positiveOutput, output, countAdd, countMult]


def karatsuba(inputX, positiveX, inputY, positiveY, b, countAdd = 0, countMult = 0):
    positiveOutput = not (positiveX ^ positiveY)

    i = 0;
    while i < (len(inputX)-1):
        if (inputX[i] == 0):
            inputX.pop(0)
            continue
        break

    i = 0;
    while i < (len(inputY)-1):
        if (inputY[i] == 0):
            inputY.pop(0)
            continue
        break

    if (len(inputX) <= 1 or len(inputY) <= 1):
        [positive, output, countAdd, countMult] = multiplication(inputX, positiveX, inputY, positiveY, b, countAdd, countMult)
        return [positive, output, countAdd, countMult]

    inputXY = inputX, inputY
    maxLen = max(len(inputX), len(inputY))

    for digits in inputXY:
        while len(digits) < maxLen:
            digits.insert(0, 0)

    if (len(inputX) % 2 == 1):
        inputX.insert(0, 0)

    if (len(inputY) % 2 == 1):
        inputY.insert(0, 0)

    n = max(len(inputX), len(inputY)) // 2

    # define x_hi, x_lo, y_hi and y_lo
    x_hi = inputX[:n]
    x_lo = inputX[n:]
    y_hi = inputY[:n]
    y_lo = inputY[n:]

    # calculate all multiplications with recursion
    [xy_hi, countAdd, countMult] = karatsuba(x_hi, True, y_hi, True, b, countAdd, countMult)[1:4]
    [xy_lo, countAdd, countMult] = karatsuba(x_lo, True, y_lo, True, b, countAdd, countMult)[1:4]
    [sumX, countAdd, countMult] = addition(x_hi, True, x_lo, True, b, countAdd, countMult)[1:4]
    [sumY, countAdd, countMult] = addition(y_hi, True, y_lo, True, b, countAdd, countMult)[1:4]
    [sumXY, countAdd, countMult] = karatsuba(sumX, True, sumY, True, b, countAdd, countMult)[1:4]
    [xy_positive, xy_temp, countAdd, countMult] = subtraction(sumXY, True, xy_hi, True, b, countAdd, countMult)
    [positiveMix, xy_mix, countAdd, countMult] = subtraction(xy_temp, xy_positive, xy_lo, True, b, countAdd, countMult)
    output = [0] * n * 10

    for i in range(1, len(xy_lo) + 1):
        output[-i] += xy_lo[-i]
        countAdd += 1

    for i in range(1, len(xy_mix) + 1):
        if positiveMix:
            output[-(i + n)] += xy_mix[-i]
        else:
            output[-(i + n)] -= xy_mix[-i]
        countAdd += 1

    for i in range(1, len(xy_hi) + 1):
        output[-(i + 2 * n)] += xy_hi[-i]
        countAdd += 1

    for i in range(1, len(output) + 1):
        rem = 0
        if (output[-i] >= b):
            carry = output[-i] // b
            output[-i] = output[-i] % b
            output[-(i + 1)] += carry
            countAdd += 1
        elif (output[-i] < 0):
            output[-(i + 1)] -= 1
            rem = output[-i] + b
            output[-i] = rem
            countAdd += 2

    i = 0
    while i < (len(output) - 1):
        if (output[i] == 0):
            output.pop(0)
            continue
        break

    return [positiveOutput, output, countAdd, countMult]

def euclidean(inputX, positiveX, inputY, positiveY, b):
    x1, x2, y1, y2 = [True, [1]], [True, [0]], [True, [0]], [True, [1]]
    eqZero = False
    numberX, numberY = inputX.copy(), inputY.copy()

    while not eqZero:
        [r, q] = modular_reduction(numberX, True, numberY, b)[1:3]
        numberX, numberY = numberY, r
        qx2 = karatsuba(q, True, x2[1], x2[0], b)[0:2]
        qy2 = karatsuba(q, True, y2[1], y2[0], b)[0:2]
        x3 = subtraction(x1[1], x1[0], qx2[1], qx2[0], b)[0:2]
        y3 = subtraction(y1[1], y1[0], qy2[1], qy2[0], b)[0:2]
        x1, y1 = x2, y2
        x2, y2 = x3, y3

        i = 0;
        while i < (len(numberY)-1):
            if (numberY[i] == 0):
                numberY.pop(0)
                continue
            break

        if (numberY[0] == 0):
            eqZero = True

    gcd = numberX

    if positiveX:
        x = x1
    else:
        x = [not x1[0], x1[1]]

    if positiveY:
        y = y1
    else:
        y = [not y1[0], y1[1]]

    return [gcd, x, y]


def modular_reduction(inputX, positiveX, m, b):
    # len of input X and len of mod
    lenX = len(inputX)
    lenM = len(m)
    difLen = lenX - lenM
    coefficient = [0]*lenX
    positive = True
    eqZero = False
    output = inputX.copy()

    # step 2
    for i in range(0, difLen + 1):
        positive = True
        coeffCounter = -1
        n = difLen-i
        tempX = output.copy()
        modBase = m.copy()
        for j in range(n):
            modBase.append(0)
        while positive:
            coeffCounter += 1
            output = tempX.copy()
            [positive, tempX] = subtraction(output, True, modBase, True, b)[0:2] #pak de positive output ding
        coefficient[-(n+1)] = coeffCounter

    i = 0;
    while i < (len(output)-1):
        if (output[i] == 0):
            output.pop(0)
            continue
        break

    if (output[0] == 0):
        eqZero = True

    if (positiveX or eqZero):
        return [True, output, coefficient]
    else:
        return [True, subtraction(m, True, output, True, b)[1], coefficient]


def modular_addition(inputX, positiveX, inputY, positiveY, m, b):
    [positive, sumXY] = addition(inputX, positiveX, inputY, positiveY, b)[0:2]
    output = modular_reduction(sumXY, positive, m, b)[0:2]
    return output


def modular_subtraction(inputX, positiveX, inputY, positiveY, m, b):
    [positive, diffXY] = subtraction(inputX, positiveX, inputY, positiveY, b)[0:2]
    output = modular_reduction(diffXY, positive, m, b)[0:2]
    return output


def modular_multiplication(inputX, positiveX, inputY, positiveY, m, b):
    [positive, prodXY] = multiplication(inputX, positiveX, inputY, positiveY, b)[0:2]
    output = modular_reduction(prodXY, positive, m, b)[0:2]
    return output


def modular_inversion(inputX, positiveX, m , b):
    a = modular_reduction(inputX, positiveX, m, b)[1]
    [gcd, x] = euclidean(a, True, m, True, b)[0:2]

    if (gcd == [1]):
        return modular_reduction(x[1], x[0], m, b)[0:2]
    else:
        return "inverse does not exist"



# -*- coding: utf-8 -*-
from distutils.core import setup

package_dir = \
{'': 'src'}

packages = \
['2wf90_assignment']

package_data = \
{'': ['*'], '2wf90_assignment': ['unused/*']}

entry_points = \
{'console_scripts': ['2wf90 = 2wf90-assignment:main']}

setup_kwargs = {
    'name': '2wf90-assignment',
    'version': '1.0.6',
    'description': 'Our attempt at the first software assignment.',
    'long_description': None,
    'author': 'Your Name',
    'author_email': 'you@example.com',
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



# Use like this: "from classes import Integer".

class Integer:
    """
    Class used to store large integers. Detailed documentation is given below,
    so here are some common use cases:

        from Integer import *

        i = Integer('6A', 16, False)
        i.digits   -> [6, 10]
        i.radix     -> 16
        i.positive -> False

        i.digits[0] = 1
        print(i) -> '-1A'
        repr(i)  -> '-[1, 10] (radix 16)'

        i = -i
        print(i) -> '1A'

        len(i) -> 2
        i = i.pad(6)
        repr(i) -> +[0, 0, 0, 0, 1, 10] (radix 16)
        len(i) -> 6

        j = Integer([1, 10, 2, 13], 16)
        print(j) -> '1A2D'
    """
    def __init__(self, digits, radix = 10, positive = True):
        """
        Arguments:
            digits:
                A list of numbers - they should all be lower than the radix.
                The most significant numbers are at the beginning of this list.
                A string can be used as well, e.g. '13AF'.
            radix:
                radix of the number. Decimal by default.
            positive:
                True if the number has a positive sign, False if negative.
                Positive by default.

        Examples:
            Integer([1, 0, 14, 2], 16, False)  -> -10E2   (radix 16)
            Integer('-100101', 2)              -> -100101 (radix 2)
            Integer([1, 2, 3])                 -> +123    (radix 10)
        """

        if radix > 16:
            raise ValueError(f'radix is {radix}. radixs beyond 16 are not supported.')

        if radix < 1:
            raise ValueError(f'radix is {radix}. radix must be positive.')

        if len(digits) == 0:
            raise ValueError('Digit list cannot be empty.')

        # Sanity checks when digits are given as a list.
        if type(digits) is list:
            for digit in digits:
                if digit >= radix:
                    raise ValueError(f'Digit {digit} is not allowed in radix {radix}.')

                if (digit < 0):
                    raise ValueError(f'Digit {digit} may not be negative.')

        # Sanity checks when digits are given as a string.
        # Also transforms string into its list representation.
        if type(digits) is str:
            if digits[0] == '-':
                positive = False
                digits = digits[1:]

            new_digits = []

            for digit in digits:
                num = char_to_number(digit)
                if num >= radix:
                    raise ValueError(f'Digit {digit} is not allowed in radix {radix}.')
                new_digits.append(num)

            digits = new_digits

        self.digits = digits
        self.radix = radix
        self.positive = positive

        # We could remove leading zeroes here using self.strip(), but for
        # assignment 1 that shouldn't be neccesary.

    def __len__(self):
        """
        Useful shortcut. Example:

            i = new Integer([1, 2, 3], 8)
            len(i) -> 3
            len(i) == len(i.digits) -> True
        """
        return len(self.digits)

    def __neg__(self):
        """
        Negation operator. Example:

            i = new Integer('ABC', 16)
            i -> +[10, 11, 12] (radix 16)
            -i -> -[10, 11, 12] (radix 16)
        """
        new_int = self.copy()
        new_int.positive = not new_int.positive
        return new_int

    def __str__(self):
        """
        Represent integer as a string. The radix is not given, and a minus sign
        is added if negative. This is the same as notation found in input.

        Examples:
            str(Integer('6A', 16, False)) -> -6A
        """
        return ('' if self.positive else '-') + f'{self.get_digit_string()}'

    def __repr__(self):
        """
        Represent as string, but with more information. Useful for debugging.
        The digit list is always printed, along with the sign and radix.

        Examples:
            repr(Integer('6A', 16, False)) -> -[6, 10] (radix 16)
        """
        return ('+' if self.positive else '-') + f'{self.digits} (radix {self.radix})'

    def __abs__(self):
        if self.positive:
            return self.copy()
        else:
            return -self

    def pad(self, size):
        """
        Add leading zeroes to the digit list to ensure it becomes a certain size.
        Returns a new copy of the Integer, so that the original one isn't modified.

        Example:
            i = Integer([1])
            i = i.pad(4)
            i.digits -> [0, 0, 0, 1]
        """
        original_size = len(self.digits)

        if size < original_size:
            raise ValueError(f'New size {size} is below original size {original_size}')

        new_int = self.copy()
        new_int.digits = [0] * (size - original_size) + new_int.digits
        return new_int

    def strip(self):
        """
        Remove trailing zeroes from the digit list. Undoes Integer.pad().
        Also called in the constructor.
        """
        new_int = self.copy()
        while new_int.digits[0] == 0 and len(new_int.digits) > 1:
            new_int.digits = new_int.digits[1:]
        return new_int

    def get_digit_string(self):
        """
        Returns the integer's digits as a string, not a list. It converts digits
        above 9 to the hexadecimal counterparts.

        Used by the class internally. Don't call it directly, but use print(i)
        instead.
        """
        s = ''
        for digit in self.digits:
            s = s + number_to_char(digit)
        return s

    def copy(self):
        """
        Returns a distinct copy of itself.
        """
        digits_copy = self.digits.copy()
        return Integer(digits_copy, self.radix, self.positive)

def char_to_number(character):
    character = character.upper()
    index = '0123456789ABCDEF'.find(character)

    if index == -1:
        raise ValueError('Character must be hexadecimal notation.')

    return index

def number_to_char(number):
    return '0123456789abcdef'[number] # ;)
